import express from "express";
import {Contract, Provider, Wallet} from "zksync-web3";
import {utils} from "ethers";
import swapAbi from "./abi/swapAbi.js";
import mathUtil from "../utils/mathUtil.js";


const router = express.Router();
router.post("/", async (req, res) => {


    const params = req.body.params;
    const amountMin = params.amountMin;
    const amountMax = params.amountMax;
    const accuracy = params.accuracy;
    const amountNumber = mathUtil.getRandomNumber(amountMin, amountMax, accuracy);
    const signer = req.body.wallet;
    const provider = new Provider('https://mainnet.era.zksync.io');

    const wallet = new Wallet(signer.privateKey, provider);
    const router = new Contract("0x8B791913eB07C32779a16750e3868aA8495F5964", swapAbi, wallet);
    const path = ["0x5AEa5775959fBC2557Cc8789bC1bf90A239D9a91", "0x3355df6D4c9C3035724Fd0e3914dE96A5a83aaf4"];
    const stable = [true, false]
    const amount = utils.parseEther(`${amountNumber}`);
    const amountsOut = await router.getAmountsOut(amount, path, stable);
    const amountOut = (amountsOut[1].toNumber() * 995 / 1000).toFixed(0);
    console.log(amountOut)
    let timestamp = new Date().getTime();
    timestamp = (timestamp / 1000 + 20 * 60).toFixed(0);
    const swapRes = await router.swapExactETHForTokensSupportingFeeOnTransferTokens(amountOut, path, signer.address, timestamp, stable, {
        value: amount
    })
    await swapRes.wait();
    const txReceipt = await provider.getTransactionReceipt(swapRes.hash);
    const gas = utils.formatEther((txReceipt.gasUsed * swapRes.gasPrice).toFixed(0));
    const balance = await wallet.getBalance()
    const balanceStr = utils.formatEther(balance);
    const value = utils.formatEther(swapRes.value)
    res.send({
        code: 0,
        msg: "",
        data: {
            txId: swapRes.hash,
            gas: gas,
            currentBalance: balanceStr,
            status: txReceipt.status,
            value: value
        }
    });


})

export default router;
