package com.ichaoj.ams.mapper;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ichaoj.ams.entity.AmsTradeRecord;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.ichaoj.ams.request.record.PageTradeRecordRequest;
import com.ichaoj.ams.response.statistics.DailyCostResponse;
import com.ichaoj.ams.response.record.TradeRecordResponse;
import com.ichaoj.ams.response.statistics.ExportResponse;
import org.apache.ibatis.annotations.Param;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 交易 Mapper 接口
 * </p>
 *
 * @author cjwen
 * @since 2023-05-18
 */
public interface AmsTradeRecordMapper extends BaseMapper<AmsTradeRecord> {

    /**
     * 分页查询交易记录
     *
     * @param buildPageObj 分页对象
     * @param request      请求参数
     * @param userId
     * @return 分页结果
     */
    Page<TradeRecordResponse> pageTradeRecord(Page<AmsTradeRecord> buildPageObj, @Param("request") PageTradeRecordRequest request, @Param("userId") String userId);

    /**
     * 统计每日消耗gas和本金
     *
     * @param startTime
     * @param endTime
     * @param userId
     * @return 统计数据
     */
    List<DailyCostResponse> dailyCostStatistics(@Param("startTime") LocalDateTime startTime, @Param("endTime") LocalDateTime endTime, @Param("userId") String userId);

    /**
     * 查询报表数据
     *
     * @param userId 用户id
     * @return 报表数据
     */
    List<ExportResponse> getExportData(String userId);

    /**
     * 根据用户id查询用户投入列表
     *
     * @param userId 用户
     * @return 列表
     */
    List<TradeRecordResponse> listByUserId(@Param("userId") String userId);
}
