package com.ichaoj.ams.script;


import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.StrUtil;
import com.ichaoj.ams.script.model.AirdropParam;
import com.ichaoj.ams.script.model.AirdropWallet;
import com.ichaoj.ams.script.model.SResult;
import com.ichaoj.ams.script.util.Web3Util;
import lombok.SneakyThrows;
import org.jetbrains.annotations.NotNull;
import org.web3j.crypto.Credentials;
import org.web3j.crypto.RawTransaction;
import org.web3j.crypto.TransactionEncoder;
import org.web3j.protocol.Web3j;
import org.web3j.protocol.core.DefaultBlockParameterName;
import org.web3j.protocol.core.Response;
import org.web3j.protocol.core.methods.response.EthSendTransaction;
import org.web3j.protocol.core.methods.response.EthTransaction;
import org.web3j.protocol.core.methods.response.Transaction;
import org.web3j.protocol.core.methods.response.TransactionReceipt;
import org.web3j.utils.Convert;
import org.web3j.utils.Numeric;

import java.io.IOException;
import java.math.BigInteger;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 脚本接口
 *
 * @author wren
 */
public abstract class JavaScript implements IScript {

    @SneakyThrows
    protected SResult sendTxForEvm(AirdropWallet airdropWallet, Web3j web3j, RawTransaction etherTransaction) {
        Credentials credentials = Web3Util.getCredentials(airdropWallet);

        byte[] signMessage = TransactionEncoder.signMessage(etherTransaction, credentials);
        String hexValue = Numeric.toHexString(signMessage);
        EthSendTransaction ethSendTransaction = web3j.ethSendRawTransaction(hexValue).send();
        return waitTxResult(airdropWallet, web3j, ethSendTransaction);
    }

    @NotNull
    protected SResult waitTxResult(AirdropWallet airdropWallet, Web3j web3j, EthSendTransaction ethSendTransaction) throws IOException {
        Response.Error error = ethSendTransaction.getError();
        if (error != null) {
            throw new RuntimeException(error.getMessage());
        }
        TransactionReceipt receipt = null;
        long currentTimeMillis = System.currentTimeMillis();
        while (true) {
            if (currentTimeMillis + 600000 < System.currentTimeMillis()) {
                throw new RuntimeException("transaction time out");
            }
            Optional<TransactionReceipt> receiptOptional = web3j
                    .ethGetTransactionReceipt(ethSendTransaction.getTransactionHash()).send().getTransactionReceipt();
            if (receiptOptional.isPresent()) {
                receipt = receiptOptional.get();
                break;
            } else {
                ThreadUtil.sleep(3, TimeUnit.SECONDS);
            }
        }
        EthTransaction ethTransaction = web3j.ethGetTransactionByHash(receipt.getTransactionHash()).send();
        Transaction transaction = ethTransaction.getTransaction().get();

        BigInteger gasAmount = receipt.getGasUsed().multiply(transaction.getGasPrice());
        SResult sResult = new SResult();
        BigInteger balance = web3j.ethGetBalance(airdropWallet.getAddress(), DefaultBlockParameterName.LATEST).send().getBalance();
        sResult.setGas(Convert.fromWei(gasAmount.toString(), Convert.Unit.ETHER).toString());
        sResult.setTxId(transaction.getHash());
        sResult.setStatus("0x1".equals(receipt.getStatus()) ? 1 : 2);
        sResult.setCurrentBalance(Convert.fromWei(balance.toString(), Convert.Unit.ETHER).toString());
        BigInteger value = transaction.getValue();
        if (value == null || value.equals(BigInteger.ZERO)) {
            sResult.setValue("0");
        } else {
            sResult.setValue(Convert.fromWei(value.toString(), Convert.Unit.ETHER).toString());
        }
        return sResult;
    }


}
